@@ -42,11 +42,20 @@ class ToPred a where toPred :: a -> Pred
 class ToTyVars a where toTyVars :: a -> [TyVarBndr]
 #if MIN_VERSION_haskell_src_exts(1,18,0)
 class ToMaybeKind a where toMaybeKind :: a -> Maybe Kind
+#endif
 #if MIN_VERSION_template_haskell(2,11,0)
 class ToInjectivityAnn a where toInjectivityAnn :: a -> InjectivityAnn
 #endif
+
+#if MIN_VERSION_template_haskell(2,12,0)
+#elif MIN_VERSION_template_haskell(2,11,0)
+type DerivClause = Pred
+#else
+type DerivClause = Name
 #endif
 
+class ToDerivClauses a where toDerivClauses :: a -> [DerivClause]
+
 -- for error messages
 moduleName = "Language.Haskell.Meta.Syntax.Translate"
 
@@ -342,7 +351,6 @@ instance ToType (Hs.Type l) where
 
 
 toStrictType :: Hs.Type l -> StrictType
-#if MIN_VERSION_haskell_src_exts(1,18,0)
 #if MIN_VERSION_template_haskell(2,11,0)
 toStrictType (Hs.TyBang _ s u t) = (Bang (toUnpack u) (toStrict s), toType t)
     where
@@ -353,7 +361,7 @@ toStrictType (Hs.TyBang _ s u t) = (Bang (toUnpack u) (toStrict s), toType t)
       toUnpack (Hs.NoUnpack _) = SourceNoUnpack
       toUnpack (Hs.NoUnpackPragma _) = NoSourceUnpackedness
 toStrictType x = (Bang NoSourceUnpackedness NoSourceStrictness, toType x)
-#else
+#elif MIN_VERSION_haskell_src_exts(1,18,0)
 -- TyBang l (BangType l) (Unpackedness l) (Type l)
 -- data BangType l = BangedTy l	| LazyTy l | NoStrictAnnot l
 -- data Unpackedness l = Unpack l | NoUnpack l | NoUnpackPragma l
@@ -364,25 +372,12 @@ toStrictType (Hs.TyBang _ b u t) = (toStrict b u, toType t)
       toStrict _ (Hs.Unpack _) = Unpacked
       toStrict _ _ = NotStrict
 toStrictType x = (NotStrict, toType x)
-#endif
 #else
-#if MIN_VERSION_template_haskell(2,11,0)
-toStrictType (Hs.TyBang _ (Hs.UnpackedTy _) t) = toStrictType2 SourceUnpack t
-toStrictType t = toStrictType2 NoSourceUnpackedness t
-
-toStrictType2 u t@(Hs.TyBang _ _ Hs.TyBang{}) =
-  nonsense "toStrictType" "double strictness annotation" t
-toStrictType2 u (Hs.TyBang _ (Hs.BangedTy _) t) = (Bang u SourceStrict, toType t)
-toStrictType2 u (Hs.TyBang _ (Hs.UnpackedTy _) t) =
-  nonsense "toStrictType" "double unpackedness annotation" t
-toStrictType2 u t = (Bang u NoSourceStrictness, toType t)
-#else /* !MIN_VERSION_template_haskell(2,11,0) */
 toStrictType t@(Hs.TyBang _ _ Hs.TyBang{}) =
   nonsense "toStrictType" "double strictness annotation" t
 toStrictType (Hs.TyBang _ (Hs.BangedTy _) t) = (IsStrict, toType t)
 toStrictType (Hs.TyBang _ (Hs.UnpackedTy _) t) = (Unpacked, toType t)
 toStrictType t = (NotStrict, toType t)
-#endif /* !MIN_VERSION_template_haskell(2,11,0) */
 #endif
 
 
@@ -402,16 +397,20 @@ instance ToPred (Hs.Asst l) where
     toPred a@Hs.IParam{} = noTH "toCxt" a
     toPred p = todo "toPred" p
 
-#if MIN_VERSION_template_haskell(2,11,0)
-instance ToCxt (Hs.Deriving l) where
-  toCxt (Hs.Deriving _ rule) = toCxt rule
-instance ToCxt [Hs.InstRule l] where
-  toCxt = concatMap toCxt
+#if MIN_VERSION_template_haskell(2,12,0)
+instance ToDerivClauses (Hs.Deriving l) where
+  toDerivClauses (Hs.Deriving _ irules) = [DerivClause Nothing (map toType irules)]
+#elif MIN_VERSION_template_haskell(2,11,0)
+instance ToDerivClauses (Hs.Deriving l) where
+  toDerivClauses (Hs.Deriving _ irules) = map toType irules
+#else
+instance ToDerivClauses (Hs.Deriving l) where
+  toDerivClauses (Hs.Deriving _ irules) = concatMap toNames irules
 #endif
 
-instance ToCxt a => ToCxt (Maybe a) where
-    toCxt Nothing = []
-    toCxt (Just a) = toCxt a
+instance ToDerivClauses a => ToDerivClauses (Maybe a) where
+  toDerivClauses Nothing  = []
+  toDerivClauses (Just a) = toDerivClauses a
 
 foldAppT :: Type -> [Type] -> Type
 foldAppT t ts = foldl' AppT t ts
@@ -444,13 +443,7 @@ instance ToDec (Hs.Decl l) where
                              Nothing
 #endif
                              (fmap qualConDeclToCon qcds)
-#if MIN_VERSION_template_haskell(2,11,0)
-                             -- Convert a Deriving into a list of types, one for each derived class
-                             -- Assumes that the types do not have any contexts
-                             (maybe [] (\(Hs.Deriving _ q) -> map toType q) qns)
-#else
-                             (toNames qns)
-#endif
+                             (toDerivClauses qns)
         Hs.NewType _  -> let qcd = case qcds of
                                      [x] -> x
                                      _   -> nonsense "toDec" ("newtype with " ++
@@ -462,11 +455,7 @@ instance ToDec (Hs.Decl l) where
                                     Nothing
 #endif
                                     (qualConDeclToCon qcd)
-#if MIN_VERSION_template_haskell(2,11,0)
-                                    (maybe [] (\(Hs.Deriving _ q) -> map toType q) qns)
-#else
-                                    (toNames qns)
-#endif
+                                    (toDerivClauses qns)
 
   -- This type-signature conversion is just wrong.
   -- Type variables need to be dealt with. /Jonas
@@ -483,7 +472,6 @@ instance ToDec (Hs.Decl l) where
     inline | b = Inline | otherwise = NoInline
 
 #if MIN_VERSION_template_haskell(2,11,0)
-#if MIN_VERSION_haskell_src_exts(1,18,0)
   toDec (Hs.TypeFamDecl _ h sig inj)
     = OpenTypeFamilyD $ TypeFamilyHead (toName h)
                                        (toTyVars h)
@@ -491,19 +479,7 @@ instance ToDec (Hs.Decl l) where
                                        (fmap toInjectivityAnn inj)
   toDec (Hs.DataFamDecl _ _ h sig)
     = DataFamilyD (toName h) (toTyVars h) (toMaybeKind sig)
-#else
-  toDec (Hs.TypeFamDecl _ h k)
-    = OpenTypeFamilyD $ TypeFamilyHead (toName h)
-                                       (toTyVars h)
-                                       (maybe NoSig (KindSig . toKind) k)
-                                       Nothing
-  -- TODO: do something with context?
-  toDec (Hs.DataFamDecl _ _ h k)
-    = DataFamilyD (toName h) (toTyVars h) (fmap toKind k)
-#endif
-
-#else
-#if MIN_VERSION_haskell_src_exts(1,18,0)
+#elif MIN_VERSION_haskell_src_exts(1,18,0)
   toDec (Hs.TypeFamDecl _ h sig inj)
     = FamilyD TypeFam (toName h) (toTyVars h) (toMaybeKind sig)
   toDec (Hs.DataFamDecl _ _ h sig)
@@ -516,7 +492,6 @@ instance ToDec (Hs.Decl l) where
   toDec (Hs.DataFamDecl _ _ h k)
     = FamilyD DataFam (toName h) (toTyVars h) (fmap toKind k)
 #endif
-#endif /* MIN_VERSION_template_haskell(2,11,0) */
 
   toDec a@(Hs.FunBind _ mtchs)                           = hsMatchesToFunD mtchs
   toDec (Hs.PatBind _ p rhs bnds)                      = ValD (toPat p)
@@ -561,12 +536,12 @@ instance ToMaybeKind (Hs.ResultSig l) where
 instance ToMaybeKind a => ToMaybeKind (Maybe a) where
     toMaybeKind Nothing = Nothing
     toMaybeKind (Just a) = toMaybeKind a
+#endif
 
 #if MIN_VERSION_template_haskell(2,11,0)
 instance ToInjectivityAnn (Hs.InjectivityInfo l) where
   toInjectivityAnn (Hs.InjectivityInfo _ n ns) = InjectivityAnn (toName n) (fmap toName ns)
 #endif
-#endif
 
 transAct :: Maybe (Hs.Activation l) -> Phases
 transAct Nothing = AllPhases
@@ -610,6 +585,10 @@ instance ToCxt (Hs.Context l) where
               Hs.CxSingle _ x' -> [toPred x']
               Hs.CxTuple _ xs -> fmap toPred xs
 
+instance ToCxt a => ToCxt (Maybe a) where
+    toCxt Nothing = []
+    toCxt (Just a) = toCxt a
+
 instance ToType (Hs.InstRule l) where
     toType (Hs.IRule _ _ _ h) = toType h
     toType (Hs.IParen _ irule) = toType irule
